"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HandlersGenerator = void 0;
const types_1 = require("../utils/types");
const openapi_1 = require("../utils/openapi");
const handlers_generator_constants_1 = require("./handlers-generator.constants");
class HandlersGenerator {
    constructor(openapiReader, openapiParser, openapiFilePath) {
        this.openapiReader = openapiReader;
        this.openapiParser = openapiParser;
        this.openapiFilePath = openapiFilePath;
    }
    generateHandlers() {
        const openapiRaw = this.openapiReader.readOpenapi(this.openapiFilePath);
        const openapi = this.openapiParser.parseOpeanpi(openapiRaw);
        const handlersProperties = this.getHandlersProperties(openapi);
        const operations = handlersProperties.map(({ operationId, summary }) => {
            const routeString = this.getRouteGenericString(operationId);
            const handler = `${operationId}?: ${routeString}`;
            const documentation = this.createDocumentation(summary);
            return `${documentation}\n\t${handler}`;
        });
        return this.formatHandlers(operations);
    }
    createDocumentation(summary) {
        if (summary != undefined &&
            summary != null &&
            typeof summary == 'string')
            return `
  /**
   * ${summary}
   */`;
        return '';
    }
    getRouteGenericProp(routeComponentType) {
        switch (routeComponentType) {
            case types_1.RouteTypeTag.PathParams:
                return 'Params';
            case types_1.RouteTypeTag.QueryParams:
                return 'Querystring';
            case types_1.RouteTypeTag.RequestBody:
                return 'Body';
            case types_1.RouteTypeTag.ReplyBody:
                return 'Reply';
        }
    }
    getRouteGeneric(operationId) {
        const routeGenricDefinition = handlers_generator_constants_1.routeTypeTags
            .map(tag => this.getRouteGenricDefinition(tag, operationId))
            .join('\n');
        return `{
${routeGenricDefinition}
    }`;
    }
    getRouteGenricDefinition(tag, operationId) {
        const routeProp = this.getRouteGenericProp(tag);
        return `\t\t\t${routeProp}: ${handlers_generator_constants_1.openapiSchemasImportName}.${operationId}${tag}`;
    }
    getRouteGenericString(operationId) {
        return `RouteHandlerMethod<
    RawServerDefault,
    RawRequestDefaultExpression<RawServerDefault>,
    RawReplyDefaultExpression<RawServerDefault>,
    ${this.getRouteGeneric(operationId)}
  >`;
    }
    getHandlersProperties(openapi) {
        if (openapi.paths === undefined)
            return [];
        return Object.values(openapi.paths).flatMap(path => openapi_1.methods.flatMap(method => {
            if (path != undefined) {
                if (method in path) {
                    const operation = path[method];
                    if (operation) {
                        const { operationId, summary } = operation;
                        if (operationId)
                            return { operationId, summary };
                    }
                }
            }
            return [];
        }));
    }
    formatHandlers(operations) {
        const operationsFormatted = operations.reduce((prev, curr) => `${prev}\n${curr}`);
        return `${handlers_generator_constants_1.disableLinter}

${handlers_generator_constants_1.fastifyImports}
${handlers_generator_constants_1.openapiSchemasImport}


${handlers_generator_constants_1.handlersInterfacePrefix}
${operationsFormatted}
${handlers_generator_constants_1.handlersInterfacePostfix}
`;
    }
}
exports.HandlersGenerator = HandlersGenerator;
